;===============================================================================
; Program Information
;===============================================================================

    ; Program:      Collect2
    ; Program by:   Darrell Spice, Jr
    ; Last Update:  March 29, 2015
    ;
    ; Super simple game of "collect the boxes" used to show how to develop
    ; a DPC+ with ARM Atari 2600 homebrew game.
    ;
    ; See readme.txt for build instructions
    
    
    
    
;===============================================================================
; Change Log
;===============================================================================

    ; 2015.03.29 - initial version.
    
    
    
    
;===============================================================================
; Initialize dasm
;===============================================================================

        PROCESSOR 6502
        include vcs.h       
        include macro.h
        include DPCplus.h

        
        
        
;===============================================================================
; Define custom Macros
;===============================================================================

        MAC BANKS_AND_VECTORS
        RORG $FFF6
SelectBank0 .byte $00
SelectBank1 .byte $00
SelectBank2 .byte $00
SelectBank3 .byte $00
SelectBank4 .byte $00
SelectBank5 .byte $00
;        .word InitSystem ; NMI - overlaps with SelectBank4 and 5
        .word InitSystem ; RESET
        .word InitSystem ; IRQ
        ENDM

        
        
        
;===============================================================================
; Define Zero Page RAM Usage
;----------------------------------------
; ZP RAM variables can only be seen by the 6507 CPU.
; C variables can only be seen by the ARM CPU.
; The 4K Display Data bank is used for any variables both CPUs need access to.
;===============================================================================

        SEG.U VARS
        ORG $80             
        
 echo "----",($00FE - *) , "bytes of RAM left (space reserved for 2 byte stack)"

 
 
 
;===============================================================================
; Define Start of Cartridge
;----------------------------------------
; DPC+ cartridges must start with the Harmony/Melody driver.  The driver is the
; ARM code that emulates the DPC+ coprocessor.
;===============================================================================

        SEG CODE    
        ORG $0000
    
HM_DRIVER:        
        INCBIN DPCplus20121020.arm
    ; calculate size as all following ORGs must compensate for the driver size 
HM_DRIVER_SIZE = * - HM_DRIVER      




;===============================================================================
; Bank 0 to 3 - ARM user code
;----------------------------------------
; The ARM code must start in bank 0 and will grow upward from there.  
; Banks 0 thru 4 may be used for ARM code.
; DO NOT make space for DPC+ registers as is done in banks for 6507 code. 
;===============================================================================

    ; the Atari banks of ROM begin *after* the Harmony/Melody driver, so we
    ; need to adjust ORG to compensate for the size of the DPC+ driver.
        ORG $0000 + HM_DRIVER_SIZE   
        RORG $0000
        
    ; include the custom ARM code.  This must be compiled before compiling
    ; the 6507 code by executing the MAKE command in the custom directory.
        INCBIN custom/bin/custom2.bin           
        
    ; value used in this echo depends upon how many banks are allocated
    ; for the ARM code - $1000 per bank, so $4000 for banks 0-3
        echo "------",($4000 - *) , "bytes of ARM stationary data space left"

        
        
        
;===============================================================================
; Bank 4 - 6507 code
;----------------------------------------
; I normally allocate banks 0-4 for ARM code, and just use bank 5 for 6507 code,
; but for this example I'll use bank 4 for the menu routines in order to include
; an example of DPC+'s bankswitching.
;===============================================================================

        ORG $4000 + HM_DRIVER_SIZE
        RORG $F000
        
    ; you can store 128 bytes of data here that's accessible to the ARM
    ; chip, but not the 6507.     
    
        echo "------",($F080 - *), " bytes free for BANK 4 'DPC+ overlap' ARM data storage"
        
    ;allocate the first $80 bytes of each 6507 bank for the DPC+ registers
        ORG $4080 + HM_DRIVER_SIZE
        RORG $F080
        
        ; menu routine will go here
                
        echo "------",($FFF6 - *) , "bytes of BANK 4 ROM left"
        ORG $4FF6 + HM_DRIVER_SIZE
        BANKS_AND_VECTORS
        
        
        
        
;===============================================================================
; Bank 5 - 6507 code
;----------------------------------------
;
;===============================================================================

        ORG $5000 + HM_DRIVER_SIZE
        RORG $F000

    ; you can store 128 bytes of data here that's accessible to the ARM
    ; chip, but not the 6507.
    
        echo "------",($F080 - *), " bytes free for BANK 4 'DPC+ overlap' ARM data storage"
        
    ;allocate the first $80 bytes of each 6507 bank for the DPC+ registers
        ORG $5080 + HM_DRIVER_SIZE
        RORG $F080
        
InitSystem:
        CLEAN_START   
        
;----------------------------------------        
; main program loop
;----------------------------------------
;Loop:
;   OverScan        - process joysticks, console switches, etc. in ARM code
;   VerticalSync    - trigger vertical sync
;   VerticalBlank   - run sprite multiplexer in ARM and populate the datastreams
;   Kernel          - draw screen using datastreams
;   goto Loop

OverScan:
        sta WSYNC       ; Wait for SYNC (halts CPU until end of scanline)
        lda #2          ; LoaD Accumulator with 2 so D1=1
        sta VBLANK      ; STore Accumulator to VBLANK, D1=1 turns image output off
        
    ; set the timer so the total number of scanlines ends up being 262
        lda #35
        sta TIM64T        
        
    ; set datafetcher 0 so we can set variables used by the ARM code  
        ldx #<DS_ToARM
        stx DF0LOW
        ldx #>DS_ToARM
        stx DF0HI
        
        ldx SWCHA           ; read state of both joysticks
        stx DF0WRITE        ; save in ARMswcha 
        ldx INPT4           ; read state of left joystick firebutton
        stx DF0WRITE        ; save in ARMinpt4 
        ldx #$FF
        stx CALLFUNCTION    ; runs main() in the C code
                
OSwait:
        sta WSYNC           ; Wait for SYNC (halts CPU until end of scanline)
        lda INTIM           ; Check the Timer
        bne OSwait          ; Branch if its Not Equal to 0
    
        
VerticalSync:
        lda #2      ; LoaD Accumulator with 2 so D1=1
        ldx #47     ; (47 * 64) / 76 = 40, used to set timer
        sta WSYNC   ; Wait for SYNC (halts CPU until end of scanline)
        sta VSYNC   ; Accumulator D1=1, turns on Vertical Sync signal
        stx TIM64T  ; set timer to go off in 40 scanlines
        sta WSYNC   ; Wait for Sync - halts CPU until end of 1st scanline of VSYNC
        sta WSYNC   ; wait until end of 2nd scanline of VSYNC
        lda #0      ; LoaD Accumulator with 0 so D1=0
        sta WSYNC   ; wait until end of 3rd scanline of VSYNC
        sta VSYNC   ; Accumulator D1=0, turns off Vertical Sync signal
            
        
VerticalBlank:

    ; set datafetcher 0 so we can retreive variables updated by the ARM code  
        ldx #<DS_FromARM
        stx DF0LOW
        ldx #>DS_FromARM
        stx DF0HI
        
        ldy DF0DATA     ; read value in ARMcolubk, Y is used in Kernel as well
        sty COLUBK      ; set the background color
        
        
        ldx #192        ; use X for KernelLoop counter
Kernel:     
        sta WSYNC
        lda INTIM 
        bne Kernel
        sta VBLANK      ; 3  9 - Accumulator D1=0, turns off Vertical Blank signal (image output on)
        
KernelLoop:
        sta WSYNC
        sty COLUBK
        dey
        dex
        bne KernelLoop
        jmp OverScan
        
        

        echo "------",($FFF6 - *) , "bytes of BANK 5 ROM left"
        ORG $5FF6 + HM_DRIVER_SIZE
        BANKS_AND_VECTORS

;===============================================================================
; Display Data
;----------------------------------------
; The Display Data bank is copied into RAM when DPC+ initializes the cartridge.
; This allows us to manipulate the data during run-time, but have a known
; starting state when the Atari is first turned on.
;
; Unlike normal Atari VCS/2600 sprite definitions, the sprite data in the
; Display Data bank is stored right-side-up.
;
; NOTE: The initial contents of Display Data is stored in ROM and can be
; accessed by ARM routines.  This can prove handy for saving ROM
;===============================================================================
        ORG $6000 + HM_DRIVER_SIZE
        RORG $0000       

DS_ToARM:        
ARMswcha:   ds 1        ; controller state to ARM code
ARMinpt4:   ds 1        ; firebutton state to ARM code

DS_FromARM:
ARMcolubk   ds 1        ; background color from ARM code
        
        if (* <= $1000)
          echo "------",($1000 - *) , "bytes of Display Data ROM left"
          ds ($1000 - *)        ; pad out remaining space in Display Data ROM

        else
          echo "FATAL ERROR - Display Data exceeds 4K", *
          err
        endif   
        
        include DPC_frequencies.h        