// Collect2 ARM routines
// Last Update:  March 29, 2015
//              Darrell Spice, Jr.
//
// types of variables, storage used and range
// const              = constant, which is compiled into the ROM section and cannot be changed during run time.
// signed char        =  8 bit, 1 byte  per value.  Range is           -128 to           127
// unsigned char      =  8 bit, 1 byte  per value.  Range is              0 to           255
// short int          = 16 bit, 2 bytes per value.  Range is        -32,768 to        32,767
// unsigned short int = 16 bit, 2 bytes per value.  Range is              0 to        65,535
// int                = 32 bit, 4 bytes per value.  Range is -2,147,483,648 to 2,147,483,647
// unsigned int       = 32 bit, 4 bytes per value.  Range is              0 to 4,294,967,295
// long int           = same as int
// float              = Not Supported
// double             = Not Supported
// *                  = 32 bit, 4 bytes, pointer (ie: char*, int*)
//
// NOTE : Only 448 bytes are allocated for use by variables (the rest of the 512 byte section is used
//        as the stack).  If you use too many variables, you'll get a compile time error:  "region ram is full"
//        You can use RAM in the Display Data to store values.
//
// NOTE : Compiled code can be significantly smaller if you use INTs for your variables instead of
//        SHORTs or CHARs.

#include "defines.h"

//#define HARMONY
#define PLUSCART
#ifdef HARMONY
	#define ARM_RAM_BASE				0x40000000
	#define ARM_RAM_DISPLAY_DATA		0x40000C00
	#define ARM_ROM_BASE				0x0000
	#define ARM_ROM_CUSTOM_CODE			0x0C00
#else
	#define STM32_OFFSET				0x590
//	#define ARM_RAM_BASE				0x10000000
	#define ARM_RAM_DISPLAY_DATA		0x10000C00
	#define ARM_ROM_BASE				(0x20000000 + STM32_OFFSET)
	#define ARM_ROM_CUSTOM_CODE			(0x20000C00 + STM32_OFFSET)
#endif


unsigned char * const queue=(unsigned char *)ARM_RAM_DISPLAY_DATA;
unsigned char * const flashdata=(unsigned char *)ARM_ROM_CUSTOM_CODE;
unsigned char * const ddrom=(unsigned char *)(ARM_ROM_BASE + 0x6C00);    // ROM copy of Display Data contents

int gFrame;

#define MAMCR *(unsigned char*)0xE01FC000

// function prototypes
void ProcessJoystick();



int main()
{
    // main() is called when the 6507 code writes to CALLFUNCTION
    
    MAMCR=1;    // partially disable MAM (cache) to stop fatal bug in ARM chip
    
    gFrame++;   // frame counter
    ProcessJoystick();
    
    MAMCR=2;    // reenable MAM
    
    return 0;
}

void ProcessJoystick()
{
    unsigned char color;
    unsigned char luma;
    
    if (!(INPT4 & 0x80))    // test if firebutton is held
    {
        if (gFrame & 0x03)  // if so, abort for 3 out of 4 frames
            return;
    }
    
    // extract current color and luma values for the background
    color   = COLUBK & 0xF0;
    luma    = COLUBK & 0x0F;
    
    if (!(SWCHA & 0x80))    // test if joystick held right
    {
        color += 0x10;
    }
    
    if (!(SWCHA & 0x40))    // test if joystick held left
    {
        color -= 16;
    }
    
    if (!(SWCHA & 0x20))    // test if joystick held down
    {
        luma--;
    }
    
    if (!(SWCHA & 0x10))     // test if joystick is held up
    {
        luma++;
    }
    
    COLUBK = color | (luma & 0x0f);
}
