/******************************************************************************
Harmony Bootloader Eeprom Loader Code - (C) Copyright 2009 - Chris Walton
******************************************************************************/

.set  FIOBASE,    0x3FFFC000    /* Fast GPIO base */
.set  SPI0BASE,   0xE0020000    /* SPI device base */
.set  EE_RDSR,    0x05          /* Read eeprom status */
.set  EE_READ,    0x03          /* Read eeprom data */

/* Put in eeprom segment */
.section  .eeprom,"ax",%progbits

/* 
ON ENTRY:
  r4 = RAM start address
  r5 = EEPROM start address
  r6 = data length
  r7 = execution start address
*/

/* EEPROM code is in THUMB mode to save space */
.global       Eeprom
.thumb
.thumb_func
Eeprom:

  /* Store start addresse */
  mov     r8, r4
  /* Store end address */
  add     r4, r4, r6
  mov     r9, r4
  /* Split EEPROM address */
  mov     r3, #0xFF
  lsr     r2, r5, #0x10
  and     r2, r3
  mov     r10, r2         /* r10 = hi */
  lsr     r2, r5, #0x08
  and     r2, r3
  mov     r11, r2         /* r11 = med */
  and     r5, r3
  mov     r12, r5         /* r12 = lo */
  
  /* Set SPI and FIO addresses */
  ldr     r0, =SPI0BASE
  ldr     r4, =FIOBASE

  /* Unmask SD card and EEPROM chip selects (mask everything else) */
  ldr     r1, =0xDFFFFF7F
  str     r1, [r4, #0x10]

  /* SPI Startup delay (100 clocks) */
  mov     r3, #0x0A
WaitSPILoop:
  mov     r1, #0xFF
  bl      ReadWriteSPI
  sub     r3, #0x01
  bne     WaitSPILoop

  /* Select EEPROM (chip select low) */
  bl      EE_Select

  /* Wait for EEPROM ready */
WaitReady:
  mov     r1, #EE_RDSR
  bl      ReadWriteSPI
  mov     r1, #0xFF
  bl      ReadWriteSPI
  mov     r2, #0x01
  and     r1, r2
  bne     WaitReady

  /* Deselect and re-select EERPROM (terminate status transmission) */
  bl      EE_Deselect
  bl      EE_Select

  /* Set EEPROM read mode */
  mov     r1, #EE_READ
  bl      ReadWriteSPI
  
  /* Set EEPROM read address */
  mov     r1, r10
  bl      ReadWriteSPI
  mov     r1, r11
  bl      ReadWriteSPI
  mov     r1, r12
  bl      ReadWriteSPI
   
  /* Copy data into RAM */
  mov     r3, r8
CopyLoop:
  mov     r1, #0xFF
  bl      ReadWriteSPI
  strb    r1, [r3, #0x00]
  add     r3, #0x01
  cmp     r3, r9
  bne     CopyLoop
  
  /* Deselect EEPROM (chip select high) */
  bl      EE_Deselect

  /* Start executing code in RAM */
  bx      r7

  /* Select EEPROM (chip select low) */
EE_Select:
  mov     r1, #0x80
  strb    r1, [r4, #0x1C]
  mov     pc, lr
  
  /* Deselect EEPROM (chip select high) */
EE_Deselect:
  mov     r1, #0x80
  strb    r1, [r4, #0x18]
  mov     pc, lr

  /* Write byte in R1 to SPI and read result */
ReadWriteSPI:
  str     r1, [r0, #0x08]
  mov     r2, #0x80
WaitSPIDone:
  ldr     r1, [r0, #0x04]
  tst     r1, r2
  beq     WaitSPIDone
  ldr     r1, [r0, #0x08]
  mov     pc, lr

  /* DEBUGGING - display LED to indicate that we have reached the end 
ShowLED:
  ldr     r0, =FIOBASE
  mov     r1, #0x20
  strb    r1, [r0, #0x03]
  mov     r1, #0xDF
  strb    r1, [r0, #0x13]
  mov     r1, #0x20
  strb    r1, [r0, #0x1F]
End:
  b       End */
  
